@@ -2,9 +2,9 @@
 
 
 import copy
-import importlib
 import logging
 import os
+import pathlib
 import sys
 import textwrap
 
@@ -12,14 +12,9 @@ import pyproject_hooks
 import pytest
 
 import build
+import build._builder
 
-
-if sys.version_info >= (3, 8):  # pragma: no cover
-    from importlib import metadata as importlib_metadata
-else:  # pragma: no cover
-    import importlib_metadata
-
-import pathlib
+from build._compat import importlib as _importlib
 
 
 build_open_owner = 'builtins'
@@ -27,11 +22,11 @@ build_open_owner = 'builtins'
 
 DEFAULT_BACKEND = {
     'build-backend': 'setuptools.build_meta:__legacy__',
-    'requires': ['setuptools >= 40.8.0', 'wheel'],
+    'requires': ['setuptools >= 40.8.0'],
 }
 
 
-class MockDistribution(importlib_metadata.Distribution):
+class MockDistribution(_importlib.metadata.Distribution):
     def locate_file(self, path):  # pragma: no cover
         return ''
 
@@ -49,7 +44,7 @@ class MockDistribution(importlib_metadata.Distribution):
             return CircularMockDistribution()
         elif name == 'nested_circular_dep':
             return NestedCircularMockDistribution()
-        raise importlib_metadata.PackageNotFoundError
+        raise _importlib.metadata.PackageNotFoundError
 
 
 class ExtraMockDistribution(MockDistribution):
@@ -168,7 +163,7 @@ class NestedCircularMockDistribution(MockDistribution):
     ],
 )
 def test_check_dependency(monkeypatch, requirement_string, expected):
-    monkeypatch.setattr(importlib_metadata, 'Distribution', MockDistribution)
+    monkeypatch.setattr(_importlib.metadata, 'Distribution', MockDistribution)
     assert next(build.check_dependency(requirement_string), None) == expected
 
 
@@ -221,20 +216,11 @@ def test_init(mocker, package_test_flit, package_legacy, test_no_permission, pac
         build.ProjectBuilder(package_test_bad_syntax)
 
 
-def test_init_makes_srcdir_absolute(package_test_flit):
+def test_init_makes_source_dir_absolute(package_test_flit):
     rel_dir = os.path.relpath(package_test_flit, os.getcwd())
     assert not os.path.isabs(rel_dir)
     builder = build.ProjectBuilder(rel_dir)
-    assert os.path.isabs(builder.srcdir)
-
-
-@pytest.mark.parametrize('value', [b'something', 'something_else'])
-def test_python_executable(package_test_flit, value):
-    builder = build.ProjectBuilder(package_test_flit)
-
-    builder.python_executable = value
-    assert builder.python_executable == value
-    assert builder._hook.python_executable == value
+    assert os.path.isabs(builder.source_dir)
 
 
 @pytest.mark.parametrize('distribution', ['wheel', 'sdist'])
@@ -373,7 +359,7 @@ def test_build_not_dir_outdir(mocker, tmp_dir, package_test_flit):
     builder._hook.build_sdist.return_value = 'dist.tar.gz'
     out = os.path.join(tmp_dir, 'out')
 
-    open(out, 'a').close()  # create empty file
+    open(out, 'a', encoding='utf-8').close()  # create empty file
 
     with pytest.raises(build.BuildException):
         builder.build('sdist', out)
@@ -383,12 +369,13 @@ def test_build_not_dir_outdir(mocker, tmp_dir, package_test_flit):
 def demo_pkg_inline(tmp_path_factory):
     # builds a wheel without any dependencies and with a console script demo-pkg-inline
     tmp_path = tmp_path_factory.mktemp('demo-pkg-inline')
-    builder = build.ProjectBuilder(srcdir=os.path.join(os.path.dirname(__file__), 'packages', 'inline'))
+    builder = build.ProjectBuilder(source_dir=os.path.join(os.path.dirname(__file__), 'packages', 'inline'))
     out = tmp_path / 'dist'
     builder.build('wheel', str(out))
     return next(out.iterdir())
 
 
+@pytest.mark.contextvars
 @pytest.mark.isolated
 def test_build_with_dep_on_console_script(tmp_path, demo_pkg_inline, capfd, mocker):
     """
@@ -398,7 +385,7 @@ def test_build_with_dep_on_console_script(tmp_path, demo_pkg_inline, capfd, mock
     # to validate backend invocations contain the correct path we use an inline backend that will fail, but first
     # provides the PATH information (and validates shutil.which is able to discover the executable - as PEP states)
     toml = textwrap.dedent(
-        '''
+        """
         [build-system]
         requires = ["demo_pkg_inline"]
         build-backend = "build"
@@ -407,20 +394,20 @@ def test_build_with_dep_on_console_script(tmp_path, demo_pkg_inline, capfd, mock
         [project]
         description = "Factory ⸻ A code generator 🏭"
         authors = [{name = "Łukasz Langa"}]
-        '''
+        """
     )
     code = textwrap.dedent(
-        '''
+        """
         import os
         import shutil
         import sys
         print("BB " + os.environ["PATH"])
         exe_at = shutil.which("demo-pkg-inline")
         print("BB " + exe_at)
-        '''
+        """
     )
     (tmp_path / 'pyproject.toml').write_text(toml, encoding='UTF-8')
-    (tmp_path / 'build.py').write_text(code)
+    (tmp_path / 'build.py').write_text(code, encoding='utf-8')
 
     deps = {str(demo_pkg_inline)}  # we patch the requires demo_pkg_inline to refer to the wheel -> we don't need index
     mocker.patch('build.ProjectBuilder.build_system_requires', new_callable=mocker.PropertyMock, return_value=deps)
@@ -472,7 +459,7 @@ def test_prepare_not_dir_outdir(mocker, tmp_dir, package_test_flit):
     builder = build.ProjectBuilder(package_test_flit)
 
     out = os.path.join(tmp_dir, 'out')
-    with open(out, 'w') as f:
+    with open(out, 'w', encoding='utf-8') as f:
         f.write('Not a directory')
     with pytest.raises(build.BuildException, match='Build path .* exists and is not a directory'):
         builder.prepare('wheel', out)
@@ -501,8 +488,9 @@ def test_no_outdir_multiple(mocker, tmp_dir, package_test_flit):
 
 
 def test_runner_user_specified(tmp_dir, package_test_flit):
-    def dummy_runner(cmd, cwd=None, env=None):
-        raise RuntimeError('Runner was called')
+    def dummy_runner(cmd, cwd=None, extra_environ=None):
+        msg = 'Runner was called'
+        raise RuntimeError(msg)
 
     builder = build.ProjectBuilder(package_test_flit, runner=dummy_runner)
     with pytest.raises(build.BuildBackendException, match='Runner was called'):
@@ -512,7 +500,7 @@ def test_runner_user_specified(tmp_dir, package_test_flit):
 def test_metadata_path_no_prepare(tmp_dir, package_test_no_prepare):
     builder = build.ProjectBuilder(package_test_no_prepare)
 
-    metadata = importlib_metadata.PathDistribution(
+    metadata = _importlib.metadata.PathDistribution(
         pathlib.Path(builder.metadata_path(tmp_dir)),
     ).metadata
 
@@ -523,18 +511,19 @@ def test_metadata_path_no_prepare(tmp_dir, package_test_no_prepare):
 def test_metadata_path_with_prepare(tmp_dir, package_test_setuptools):
     builder = build.ProjectBuilder(package_test_setuptools)
 
-    metadata = importlib_metadata.PathDistribution(
+    metadata = _importlib.metadata.PathDistribution(
         pathlib.Path(builder.metadata_path(tmp_dir)),
     ).metadata
 
-    assert metadata['name'] == 'test-setuptools'
+    # Setuptools < v69.0.3 (https://github.com/pypa/setuptools/pull/4159) normalized this to dashes
+    assert metadata['name'].replace('-', '_') == 'test_setuptools'
     assert metadata['Version'] == '1.0.0'
 
 
 def test_metadata_path_legacy(tmp_dir, package_legacy):
     builder = build.ProjectBuilder(package_legacy)
 
-    metadata = importlib_metadata.PathDistribution(
+    metadata = _importlib.metadata.PathDistribution(
         pathlib.Path(builder.metadata_path(tmp_dir)),
     ).metadata
 
@@ -549,31 +538,6 @@ def test_metadata_invalid_wheel(tmp_dir, package_test_bad_wheel):
         builder.metadata_path(tmp_dir)
 
 
-@pytest.fixture
-def mock_tomli_not_available(mocker):
-    loads = mocker.patch('tomli.loads')
-    mocker.patch.dict(sys.modules, {'tomli': None})
-    importlib.reload(build)
-    try:
-        yield
-    finally:
-        loads.assert_not_called()
-        mocker.stopall()
-        importlib.reload(build)
-
-
-@pytest.mark.skipif(sys.version_info >= (3, 11), reason='No need to test old toml support on 3.11+')
-def test_toml_instead_of_tomli(mocker, mock_tomli_not_available, tmp_dir, package_test_flit):
-    mocker.patch('pyproject_hooks.BuildBackendHookCaller', autospec=True)
-
-    builder = build.ProjectBuilder(package_test_flit)
-    builder._hook.build_sdist.return_value = 'dist.tar.gz'
-
-    builder.build('sdist', '.')
-
-    builder._hook.build_sdist.assert_called_with(os.path.abspath('.'), None)
-
-
 def test_log(mocker, caplog, package_test_flit):
     mocker.patch('pyproject_hooks.BuildBackendHookCaller', autospec=True)
     mocker.patch('build.ProjectBuilder._call_backend', return_value='some_path')
@@ -585,7 +549,6 @@ def test_log(mocker, caplog, package_test_flit):
     builder.prepare('wheel', '.')
     builder.build('sdist', '.')
     builder.build('wheel', '.')
-    builder.log('something')
 
     assert [(record.levelname, record.message) for record in caplog.records] == [
         ('INFO', 'Getting build dependencies for sdist...'),
@@ -593,7 +556,6 @@ def test_log(mocker, caplog, package_test_flit):
         ('INFO', 'Getting metadata for wheel...'),
         ('INFO', 'Building sdist...'),
         ('INFO', 'Building wheel...'),
-        ('INFO', 'something'),
     ]
 
 
@@ -615,7 +577,7 @@ def test_log(mocker, caplog, package_test_flit):
     ],
 )
 def test_parse_valid_build_system_table_type(pyproject_toml, parse_output):
-    assert build._parse_build_system_table(pyproject_toml) == parse_output
+    assert build._builder._parse_build_system_table(pyproject_toml) == parse_output
 
 
 @pytest.mark.parametrize(
@@ -653,4 +615,4 @@ def test_parse_valid_build_system_table_type(pyproject_toml, parse_output):
 )
 def test_parse_invalid_build_system_table_type(pyproject_toml, error_message):
     with pytest.raises(build.BuildSystemTableValidationError, match=error_message):
-        build._parse_build_system_table(pyproject_toml)
+        build._builder._parse_build_system_table(pyproject_toml)
